@@ -50,7 +50,7 @@
 #define NGHTTP3_QPACK_MAX_VALUELEN 65536
 
 /* nghttp3_qpack_indexing_mode is a indexing strategy. */
-typedef enum {
+typedef enum nghttp3_qpack_indexing_mode {
   /* NGHTTP3_QPACK_INDEXING_MODE_LITERAL means that header field
      should not be inserted into dynamic table. */
   NGHTTP3_QPACK_INDEXING_MODE_LITERAL,
@@ -63,7 +63,6 @@ typedef enum {
   NGHTTP3_QPACK_INDEXING_MODE_NEVER,
 } nghttp3_qpack_indexing_mode;
 
-struct nghttp3_qpack_entry;
 typedef struct nghttp3_qpack_entry nghttp3_qpack_entry;
 
 struct nghttp3_qpack_entry {
@@ -82,13 +81,13 @@ struct nghttp3_qpack_entry {
 };
 
 /* The entry used for static table. */
-typedef struct {
+typedef struct nghttp3_qpack_static_entry {
   uint64_t absidx;
   int32_t token;
   uint32_t hash;
 } nghttp3_qpack_static_entry;
 
-typedef struct {
+typedef struct nghttp3_qpack_static_header {
   nghttp3_rcbuf name;
   nghttp3_rcbuf value;
   int32_t token;
@@ -99,7 +98,7 @@ typedef struct {
  * and includes the required insert count and the minimum insert count
  * of dynamic table entry it refers to.
  */
-typedef struct {
+typedef struct nghttp3_qpack_header_block_ref {
   nghttp3_pq_entry max_cnts_pe;
   nghttp3_pq_entry min_cnts_pe;
   /* max_cnt is the required insert count. */
@@ -118,8 +117,8 @@ int nghttp3_qpack_header_block_ref_new(nghttp3_qpack_header_block_ref **pref,
 void nghttp3_qpack_header_block_ref_del(nghttp3_qpack_header_block_ref *ref,
                                         const nghttp3_mem *mem);
 
-typedef struct {
-  nghttp3_map_entry me;
+typedef struct nghttp3_qpack_stream {
+  int64_t stream_id;
   /* refs is an array of pointer to nghttp3_qpack_header_block_ref in
      the order of the time they are encoded.  HTTP/3 allows multiple
      header blocks (e.g., non-final response headers, final response
@@ -145,7 +144,7 @@ void nghttp3_qpack_stream_pop_ref(nghttp3_qpack_stream *stream);
 
 #define NGHTTP3_QPACK_ENTRY_OVERHEAD 32
 
-typedef struct {
+typedef struct nghttp3_qpack_context {
   /* dtable is a dynamic table */
   nghttp3_ringbuf dtable;
   /* mem is memory allocator */
@@ -155,18 +154,19 @@ typedef struct {
      NGHTTP3_QPACK_ENTRY_OVERHEAD bytes overhead per each entry. */
   size_t dtable_size;
   size_t dtable_sum;
-  /* hard_max_dtable_size is the maximum size of dynamic table.  In
-     HTTP/3, it is notified by decoder as
+  /* hard_max_dtable_capacity is the maximum size of dynamic table.
+     In HTTP/3, it is notified by decoder as
      SETTINGS_QPACK_MAX_TABLE_CAPACITY.  Any value lower than or equal
      to SETTINGS_QPACK_MAX_TABLE_CAPACITY is OK because encoder has
      the authority to decide how many entries are inserted into
      dynamic table. */
-  size_t hard_max_dtable_size;
-  /* max_dtable_size is the effective maximum size of dynamic table. */
-  size_t max_dtable_size;
-  /* max_blocked is the maximum number of stream which can be
+  size_t hard_max_dtable_capacity;
+  /* max_dtable_capacity is the effective maximum size of dynamic
+     table. */
+  size_t max_dtable_capacity;
+  /* max_blocked_streams is the maximum number of stream which can be
      blocked. */
-  size_t max_blocked;
+  size_t max_blocked_streams;
   /* next_absidx is the next absolute index for nghttp3_qpack_entry.
      It is equivalent to insert count. */
   uint64_t next_absidx;
@@ -176,7 +176,7 @@ typedef struct {
   uint8_t bad;
 } nghttp3_qpack_context;
 
-typedef struct {
+typedef struct nghttp3_qpack_read_state {
   nghttp3_qpack_huffman_decode_context huffman_ctx;
   nghttp3_buf namebuf;
   nghttp3_buf valuebuf;
@@ -197,33 +197,32 @@ void nghttp3_qpack_read_state_reset(nghttp3_qpack_read_state *rstate);
 
 #define NGHTTP3_QPACK_MAP_SIZE 64
 
-typedef struct {
+typedef struct nghttp3_qpack_map {
   nghttp3_qpack_entry *table[NGHTTP3_QPACK_MAP_SIZE];
 } nghttp3_qpack_map;
 
 /* nghttp3_qpack_decoder_stream_state is a set of states when decoding
    decoder stream. */
-typedef enum {
+typedef enum nghttp3_qpack_decoder_stream_state {
   NGHTTP3_QPACK_DS_STATE_OPCODE,
   NGHTTP3_QPACK_DS_STATE_READ_NUMBER,
 } nghttp3_qpack_decoder_stream_state;
 
 /* nghttp3_qpack_decoder_stream_opcode is opcode used in decoder
    stream. */
-typedef enum {
+typedef enum nghttp3_qpack_decoder_stream_opcode {
   NGHTTP3_QPACK_DS_OPCODE_ICNT_INCREMENT,
   NGHTTP3_QPACK_DS_OPCODE_SECTION_ACK,
   NGHTTP3_QPACK_DS_OPCODE_STREAM_CANCEL,
 } nghttp3_qpack_decoder_stream_opcode;
 
-/* nghttp3_qpack_encoder_flag is a set of flags used by
-   nghttp3_qpack_encoder. */
-typedef enum {
-  NGHTTP3_QPACK_ENCODER_FLAG_NONE = 0x00,
-  /* NGHTTP3_QPACK_ENCODER_FLAG_PENDING_SET_DTABLE_CAP indicates that
-     Set Dynamic Table Capacity is required. */
-  NGHTTP3_QPACK_ENCODER_FLAG_PENDING_SET_DTABLE_CAP = 0x01,
-} nghttp3_qpack_encoder_flag;
+/* QPACK encoder flags */
+
+/* NGHTTP3_QPACK_ENCODER_FLAG_NONE indicates that no flag is set. */
+#define NGHTTP3_QPACK_ENCODER_FLAG_NONE 0x00
+/* NGHTTP3_QPACK_ENCODER_FLAG_PENDING_SET_DTABLE_CAP indicates that
+   Set Dynamic Table Capacity is required. */
+#define NGHTTP3_QPACK_ENCODER_FLAG_PENDING_SET_DTABLE_CAP 0x01
 
 struct nghttp3_qpack_encoder {
   nghttp3_qpack_context ctx;
@@ -256,15 +255,13 @@ struct nghttp3_qpack_encoder {
      requested. */
   size_t last_max_dtable_update;
   /* flags is bitwise OR of zero or more of
-     nghttp3_qpack_encoder_flag. */
+     NGHTTP3_QPACK_ENCODER_FLAG_*. */
   uint8_t flags;
 };
 
 /*
- * nghttp3_qpack_encoder_init initializes |encoder|.
- * |max_dtable_size| is the maximum size of dynamic table.
- * |max_blocked| is the maximum number of stream which can be blocked.
- * |mem| is a memory allocator.
+ * nghttp3_qpack_encoder_init initializes |encoder|.  |mem| is a
+ * memory allocator.
  *
  * This function returns 0 if it succeeds, or one of the following
  * negative error codes:
@@ -273,7 +270,6 @@ struct nghttp3_qpack_encoder {
  *     Out of memory.
  */
 int nghttp3_qpack_encoder_init(nghttp3_qpack_encoder *encoder,
-                               size_t max_dtable_size, size_t max_blocked,
                                const nghttp3_mem *mem);
 
 /*
@@ -302,7 +298,7 @@ int nghttp3_qpack_encoder_encode_nv(nghttp3_qpack_encoder *encoder,
                                     int allow_blocking);
 
 /* nghttp3_qpack_lookup_result stores a result of table lookup. */
-typedef struct {
+typedef struct nghttp3_qpack_lookup_result {
   /* index is an index of matched entry.  -1 if no match is made. */
   nghttp3_ssize index;
   /* name_value_match is nonzero if both name and value are
@@ -630,6 +626,44 @@ int nghttp3_qpack_encoder_dtable_literal_add(nghttp3_qpack_encoder *encoder,
                                              const nghttp3_nv *nv,
                                              int32_t token, uint32_t hash);
 
+/*
+ * `nghttp3_qpack_encoder_ack_header` tells |encoder| that header
+ * block for a stream denoted by |stream_id| was acknowledged by
+ * decoder.
+ *
+ * This function returns 0 if it succeeds, or one of the following
+ * negative error codes:
+ *
+ * :macro:`NGHTTP3_ERR_QPACK_DECODER_STREAM_ERROR`
+ *     Section Acknowledgement for a stream denoted by |stream_id| is
+ *     unexpected.
+ */
+int nghttp3_qpack_encoder_ack_header(nghttp3_qpack_encoder *encoder,
+                                     int64_t stream_id);
+
+/*
+ * `nghttp3_qpack_encoder_add_icnt` increments known received count of
+ * |encoder| by |n|.
+ *
+ * This function returns 0 if it succeeds, or one of the following
+ * negative error codes:
+ *
+ * :macro:`NGHTTP3_ERR_NOMEM`
+ *     Out of memory.
+ * :macro:`NGHTTP3_ERR_QPACK_DECODER_STREAM_ERROR`
+ *     |n| is too large.
+ */
+int nghttp3_qpack_encoder_add_icnt(nghttp3_qpack_encoder *encoder, uint64_t n);
+
+/*
+ * `nghttp3_qpack_encoder_cancel_stream` tells |encoder| that stream
+ * denoted by |stream_id| is cancelled.  This function is provided for
+ * debugging purpose only.  In HTTP/3, |encoder| knows this by reading
+ * decoder stream with `nghttp3_qpack_encoder_read_decoder()`.
+ */
+void nghttp3_qpack_encoder_cancel_stream(nghttp3_qpack_encoder *encoder,
+                                         int64_t stream_id);
+
 /*
  * nghttp3_qpack_context_dtable_get returns dynamic table entry whose
  * absolute index is |absidx|.  This function assumes that such entry
@@ -678,7 +712,7 @@ uint8_t *nghttp3_qpack_put_varint(uint8_t *buf, uint64_t n, size_t prefix);
 
 /* nghttp3_qpack_encoder_stream_state is a set of states for encoder
    stream decoding. */
-typedef enum {
+typedef enum nghttp3_qpack_encoder_stream_state {
   NGHTTP3_QPACK_ES_STATE_OPCODE,
   NGHTTP3_QPACK_ES_STATE_READ_INDEX,
   NGHTTP3_QPACK_ES_STATE_CHECK_NAME_HUFFMAN,
@@ -693,7 +727,7 @@ typedef enum {
 
 /* nghttp3_qpack_encoder_stream_opcode is a set of opcodes used in
    encoder stream. */
-typedef enum {
+typedef enum nghttp3_qpack_encoder_stream_opcode {
   NGHTTP3_QPACK_ES_OPCODE_INSERT_INDEXED,
   NGHTTP3_QPACK_ES_OPCODE_INSERT,
   NGHTTP3_QPACK_ES_OPCODE_DUPLICATE,
@@ -702,7 +736,7 @@ typedef enum {
 
 /* nghttp3_qpack_request_stream_state is a set of states for request
    stream decoding. */
-typedef enum {
+typedef enum nghttp3_qpack_request_stream_state {
   NGHTTP3_QPACK_RS_STATE_RICNT,
   NGHTTP3_QPACK_RS_STATE_DBASE_SIGN,
   NGHTTP3_QPACK_RS_STATE_DBASE,
@@ -721,7 +755,7 @@ typedef enum {
 
 /* nghttp3_qpack_request_stream_opcode is a set of opcodes used in
    request stream. */
-typedef enum {
+typedef enum nghttp3_qpack_request_stream_opcode {
   NGHTTP3_QPACK_RS_OPCODE_INDEXED,
   NGHTTP3_QPACK_RS_OPCODE_INDEXED_PB,
   NGHTTP3_QPACK_RS_OPCODE_INDEXED_NAME,
@@ -751,9 +785,9 @@ struct nghttp3_qpack_decoder {
 
 /*
  * nghttp3_qpack_decoder_init initializes |decoder|.
- * |max_dtable_size| is the maximum size of dynamic table.
- * |max_blocked| is the maximum number of stream which can be blocked.
- * |mem| is a memory allocator.
+ * |max_dtable_capacity| is the maximum size of dynamic table.
+ * |max_blocked_streams| is the maximum number of stream which can be
+ * blocked.  |mem| is a memory allocator.
  *
  * This function returns 0 if it succeeds, or one of the following
  * negative error codes:
@@ -762,7 +796,8 @@ struct nghttp3_qpack_decoder {
  *     Out of memory.
  */
 int nghttp3_qpack_decoder_init(nghttp3_qpack_decoder *decoder,
-                               size_t max_dtable_size, size_t max_blocked,
+                               size_t max_dtable_capacity,
+                               size_t max_blocked_streams,
                                const nghttp3_mem *mem);
 
 /*
@@ -877,8 +912,6 @@ void nghttp3_qpack_stream_context_init(nghttp3_qpack_stream_context *sctx,
  */
 void nghttp3_qpack_stream_context_free(nghttp3_qpack_stream_context *sctx);
 
-void nghttp3_qpack_stream_context_reset(nghttp3_qpack_stream_context *sctx);
-
 /*
  * nghttp3_qpack_decoder_reconstruct_ricnt reconstructs Required
  * Insert Count from the encoded form |encricnt| and stores Required
@@ -939,9 +972,9 @@ void nghttp3_qpack_decoder_emit_indexed(nghttp3_qpack_decoder *decoder,
                                         nghttp3_qpack_stream_context *sctx,
                                         nghttp3_qpack_nv *nv);
 
-void nghttp3_qpack_decoder_emit_indexed_name(nghttp3_qpack_decoder *decoder,
-                                             nghttp3_qpack_stream_context *sctx,
-                                             nghttp3_qpack_nv *nv);
+int nghttp3_qpack_decoder_emit_indexed_name(nghttp3_qpack_decoder *decoder,
+                                            nghttp3_qpack_stream_context *sctx,
+                                            nghttp3_qpack_nv *nv);
 
 void nghttp3_qpack_decoder_emit_literal(nghttp3_qpack_decoder *decoder,
                                         nghttp3_qpack_stream_context *sctx,
