@@ -42,7 +42,7 @@ static int init_suite1(void) { return 0; }
 
 static int clean_suite1(void) { return 0; }
 
-int main() {
+int main(void) {
   CU_pSuite pSuite = NULL;
   unsigned int num_tests_failed;
 
@@ -79,8 +79,6 @@ int main() {
                    test_nghttp3_conn_write_control) ||
       !CU_add_test(pSuite, "conn_submit_request",
                    test_nghttp3_conn_submit_request) ||
-      !CU_add_test(pSuite, "conn_submit_push_promise",
-                   test_nghttp3_conn_submit_push_promise) ||
       !CU_add_test(pSuite, "conn_http_request",
                    test_nghttp3_conn_http_request) ||
       !CU_add_test(pSuite, "conn_http_resp_header",
@@ -101,20 +99,29 @@ int main() {
                    test_nghttp3_conn_http_record_request_method) ||
       !CU_add_test(pSuite, "conn_qpack_blocked_stream",
                    test_nghttp3_conn_qpack_blocked_stream) ||
-      !CU_add_test(pSuite, "conn_recv_cancel_push",
-                   test_nghttp3_conn_recv_cancel_push) ||
-      !CU_add_test(pSuite, "conn_cancel_push", test_nghttp3_conn_cancel_push) ||
-      !CU_add_test(pSuite, "conn_recv_push_promise",
-                   test_nghttp3_conn_recv_push_promise) ||
-      !CU_add_test(pSuite, "conn_recv_push_stream",
-                   test_nghttp3_conn_recv_push_stream) ||
       !CU_add_test(pSuite, "conn_submit_response_read_blocked",
                    test_nghttp3_conn_submit_response_read_blocked) ||
       !CU_add_test(pSuite, "conn_just_fin", test_nghttp3_conn_just_fin) ||
       !CU_add_test(pSuite, "conn_recv_uni", test_nghttp3_conn_recv_uni) ||
+      !CU_add_test(pSuite, "conn_recv_goaway", test_nghttp3_conn_recv_goaway) ||
+      !CU_add_test(pSuite, "conn_shutdown_server",
+                   test_nghttp3_conn_shutdown_server) ||
+      !CU_add_test(pSuite, "conn_shutdown_client",
+                   test_nghttp3_conn_shutdown_client) ||
+      !CU_add_test(pSuite, "conn_priority_update",
+                   test_nghttp3_conn_priority_update) ||
+      !CU_add_test(pSuite, "conn_set_stream_priority",
+                   test_nghttp3_conn_set_stream_priority) ||
+      !CU_add_test(pSuite, "conn_shutdown_stream_read",
+                   test_nghttp3_conn_shutdown_stream_read) ||
+      !CU_add_test(pSuite, "conn_stream_data_overflow",
+                   test_nghttp3_conn_stream_data_overflow) ||
       !CU_add_test(pSuite, "tnode_schedule", test_nghttp3_tnode_schedule) ||
       !CU_add_test(pSuite, "http_parse_priority",
                    test_nghttp3_http_parse_priority) ||
+      !CU_add_test(pSuite, "sf_parse_item", test_nghttp3_sf_parse_item) ||
+      !CU_add_test(pSuite, "sf_parse_inner_list",
+                   test_nghttp3_sf_parse_inner_list) ||
       !CU_add_test(pSuite, "pri_to_uint8", test_nghttp3_pri_to_uint8)) {
     CU_cleanup_registry();
     return (int)CU_get_error();
