@@ -163,11 +163,14 @@ void test_nghttp3_qpack_encoder_encode(void) {
   nghttp3_buf_init(&pbuf);
   nghttp3_buf_init(&rbuf);
   nghttp3_buf_init(&ebuf);
-  rv = nghttp3_qpack_encoder_init(&enc, 4096, 1, mem);
+  rv = nghttp3_qpack_encoder_init(&enc, mem);
 
   CU_ASSERT(0 == rv);
 
-  nghttp3_qpack_encoder_set_max_dtable_size(&enc, 4096);
+  nghttp3_qpack_encoder_set_hard_max_dtable_capacity(&enc, 4096);
+  nghttp3_qpack_encoder_set_max_blocked_streams(&enc, 1);
+
+  nghttp3_qpack_encoder_set_max_dtable_capacity(&enc, 4096);
 
   rv = nghttp3_qpack_decoder_init(&dec, 4096, 1, mem);
 
@@ -182,7 +185,7 @@ void test_nghttp3_qpack_encoder_encode(void) {
 
   CU_ASSERT(NULL != stream);
   CU_ASSERT(nghttp3_qpack_encoder_stream_is_blocked(&enc, stream));
-  CU_ASSERT(1 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(1 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
   ref =
       *(nghttp3_qpack_header_block_ref **)nghttp3_ringbuf_get(&stream->refs, 0);
@@ -216,7 +219,7 @@ void test_nghttp3_qpack_encoder_encode(void) {
                                     nghttp3_arraylen(nva));
 
   CU_ASSERT(0 == rv);
-  CU_ASSERT(0 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(0 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
   check_decode_header(&dec, &pbuf, &rbuf, &ebuf, 8, nva, nghttp3_arraylen(nva),
                       mem);
@@ -247,23 +250,26 @@ void test_nghttp3_qpack_encoder_still_blocked(void) {
   nghttp3_buf_init(&pbuf);
   nghttp3_buf_init(&rbuf);
   nghttp3_buf_init(&ebuf);
-  rv = nghttp3_qpack_encoder_init(&enc, 4096, 1, mem);
+  rv = nghttp3_qpack_encoder_init(&enc, mem);
 
   CU_ASSERT(0 == rv);
 
-  nghttp3_qpack_encoder_set_max_dtable_size(&enc, 4096);
+  nghttp3_qpack_encoder_set_hard_max_dtable_capacity(&enc, 4096);
+  nghttp3_qpack_encoder_set_max_blocked_streams(&enc, 1);
+
+  nghttp3_qpack_encoder_set_max_dtable_capacity(&enc, 4096);
 
   rv = nghttp3_qpack_encoder_encode(&enc, &pbuf, &rbuf, &ebuf, 0, nva1,
                                     nghttp3_arraylen(nva1));
 
   CU_ASSERT(0 == rv);
-  CU_ASSERT(1 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(1 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
   rv = nghttp3_qpack_encoder_encode(&enc, &pbuf, &rbuf, &ebuf, 0, nva2,
                                     nghttp3_arraylen(nva2));
 
   CU_ASSERT(0 == rv);
-  CU_ASSERT(1 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(1 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
   stream = nghttp3_qpack_encoder_find_stream(&enc, 0);
 
@@ -280,7 +286,7 @@ void test_nghttp3_qpack_encoder_still_blocked(void) {
 
   nghttp3_qpack_encoder_ack_header(&enc, 0);
 
-  CU_ASSERT(1 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(1 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
   stream = nghttp3_qpack_encoder_find_stream(&enc, 0);
 
@@ -292,7 +298,7 @@ void test_nghttp3_qpack_encoder_still_blocked(void) {
 
   nghttp3_qpack_encoder_ack_header(&enc, 0);
 
-  CU_ASSERT(0 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(0 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
   CU_ASSERT(NULL == nghttp3_qpack_encoder_find_stream(&enc, 0));
 
   nghttp3_qpack_encoder_free(&enc);
@@ -321,11 +327,14 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
   nghttp3_buf_init(&rbuf);
   nghttp3_buf_init(&ebuf);
 
-  rv = nghttp3_qpack_encoder_init(&enc, 4096, 3, mem);
+  rv = nghttp3_qpack_encoder_init(&enc, mem);
 
   CU_ASSERT(0 == rv);
 
-  nghttp3_qpack_encoder_set_max_dtable_size(&enc, 4096);
+  nghttp3_qpack_encoder_set_hard_max_dtable_capacity(&enc, 4096);
+  nghttp3_qpack_encoder_set_max_blocked_streams(&enc, 3);
+
+  nghttp3_qpack_encoder_set_max_dtable_capacity(&enc, 4096);
 
   rv = nghttp3_qpack_decoder_init(&dec, 4096, 3, mem);
 
@@ -346,7 +355,7 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
   CU_ASSERT(1 == dec.ctx.next_absidx);
   CU_ASSERT(strlen("date") + strlen("bar1") + NGHTTP3_QPACK_ENTRY_OVERHEAD ==
             dec.ctx.dtable_size);
-  CU_ASSERT(4096 == dec.ctx.max_dtable_size);
+  CU_ASSERT(4096 == dec.ctx.max_dtable_capacity);
 
   decode_header_block(&dec, &pbuf, &rbuf, 0, mem);
   nghttp3_buf_reset(&pbuf);
@@ -362,7 +371,7 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
                 NGHTTP3_QPACK_ENTRY_OVERHEAD ==
             enc.ctx.dtable_size);
   CU_ASSERT(2 == enc.ctx.next_absidx);
-  CU_ASSERT(2 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(2 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
   nread = nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos,
                                              nghttp3_buf_len(&ebuf));
@@ -373,27 +382,28 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
                 strlen("vary") + strlen("bar2") +
                 NGHTTP3_QPACK_ENTRY_OVERHEAD ==
             dec.ctx.dtable_size);
-  CU_ASSERT(4096 == dec.ctx.max_dtable_size);
+  CU_ASSERT(4096 == dec.ctx.max_dtable_capacity);
 
   decode_header_block(&dec, &pbuf, &rbuf, 4, mem);
   nghttp3_buf_reset(&pbuf);
   nghttp3_buf_reset(&rbuf);
   nghttp3_buf_reset(&ebuf);
 
-  rv = nghttp3_qpack_encoder_set_max_dtable_size(&enc, 0);
+  rv = nghttp3_qpack_encoder_set_max_dtable_capacity(&enc, 0);
 
   CU_ASSERT(0 == rv);
-  CU_ASSERT(0 == enc.ctx.max_dtable_size);
-  CU_ASSERT(2 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(0 == enc.ctx.max_dtable_capacity);
+  CU_ASSERT(2 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
-  /* Cannot index more headers because we set max_dtable_size to 0. */
+  /* Cannot index more headers because we set max_dtable_capacity to
+     0. */
   rv = nghttp3_qpack_encoder_encode(&enc, &pbuf, &rbuf, &ebuf, 8, nva2,
                                     nghttp3_arraylen(nva2));
 
   CU_ASSERT(0 == rv);
-  CU_ASSERT(0 == enc.ctx.max_dtable_size);
+  CU_ASSERT(0 == enc.ctx.max_dtable_capacity);
   CU_ASSERT(2 == enc.ctx.next_absidx);
-  CU_ASSERT(2 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(2 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
   nread = nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos,
                                              nghttp3_buf_len(&ebuf));
@@ -404,7 +414,7 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
                 strlen("vary") + strlen("bar2") +
                 NGHTTP3_QPACK_ENTRY_OVERHEAD ==
             dec.ctx.dtable_size);
-  CU_ASSERT(4096 == dec.ctx.max_dtable_size);
+  CU_ASSERT(4096 == dec.ctx.max_dtable_capacity);
 
   decode_header_block(&dec, &pbuf, &rbuf, 8, mem);
   nghttp3_buf_reset(&pbuf);
@@ -420,8 +430,8 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
   CU_ASSERT(0 == rv);
   CU_ASSERT(strlen("vary") + strlen("bar2") + NGHTTP3_QPACK_ENTRY_OVERHEAD ==
             enc.ctx.dtable_size);
-  CU_ASSERT(0 == enc.ctx.max_dtable_size);
-  CU_ASSERT(1 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(0 == enc.ctx.max_dtable_capacity);
+  CU_ASSERT(1 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
   nread = nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos,
                                              nghttp3_buf_len(&ebuf));
@@ -434,7 +444,7 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
                 strlen("vary") + strlen("bar2") +
                 NGHTTP3_QPACK_ENTRY_OVERHEAD ==
             dec.ctx.dtable_size);
-  CU_ASSERT(4096 == dec.ctx.max_dtable_size);
+  CU_ASSERT(4096 == dec.ctx.max_dtable_capacity);
 
   decode_header_block(&dec, &pbuf, &rbuf, 12, mem);
   nghttp3_buf_reset(&pbuf);
@@ -449,10 +459,10 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
 
   CU_ASSERT(0 == rv);
   CU_ASSERT(0 == enc.ctx.dtable_size);
-  CU_ASSERT(0 == enc.ctx.max_dtable_size);
+  CU_ASSERT(0 == enc.ctx.max_dtable_capacity);
   CU_ASSERT(0 == enc.last_max_dtable_update);
   CU_ASSERT(SIZE_MAX == enc.min_dtable_update);
-  CU_ASSERT(0 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(0 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
   nread = nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos,
                                              nghttp3_buf_len(&ebuf));
@@ -460,7 +470,7 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
   CU_ASSERT(nread == (nghttp3_ssize)nghttp3_buf_len(&ebuf));
   CU_ASSERT(2 == dec.ctx.next_absidx);
   CU_ASSERT(0 == dec.ctx.dtable_size);
-  CU_ASSERT(0 == dec.ctx.max_dtable_size);
+  CU_ASSERT(0 == dec.ctx.max_dtable_capacity);
 
   decode_header_block(&dec, &pbuf, &rbuf, 16, mem);
   nghttp3_buf_reset(&pbuf);
@@ -471,8 +481,10 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
   nghttp3_qpack_encoder_free(&enc);
 
   /* Check that minimum size is emitted */
-  nghttp3_qpack_encoder_init(&enc, 4096, 1, mem);
-  nghttp3_qpack_encoder_set_max_dtable_size(&enc, 4096);
+  nghttp3_qpack_encoder_init(&enc, mem);
+  nghttp3_qpack_encoder_set_hard_max_dtable_capacity(&enc, 4096);
+  nghttp3_qpack_encoder_set_max_blocked_streams(&enc, 1);
+  nghttp3_qpack_encoder_set_max_dtable_capacity(&enc, 4096);
   nghttp3_qpack_decoder_init(&dec, 4096, 1, mem);
 
   rv = nghttp3_qpack_encoder_encode(&enc, &pbuf, &rbuf, &ebuf, 0, nva1,
@@ -490,18 +502,18 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
   CU_ASSERT(1 == dec.ctx.next_absidx);
   CU_ASSERT(strlen("date") + strlen("bar1") + NGHTTP3_QPACK_ENTRY_OVERHEAD ==
             dec.ctx.dtable_size);
-  CU_ASSERT(4096 == dec.ctx.max_dtable_size);
+  CU_ASSERT(4096 == dec.ctx.max_dtable_capacity);
 
   decode_header_block(&dec, &pbuf, &rbuf, 0, mem);
   nghttp3_buf_reset(&pbuf);
   nghttp3_buf_reset(&rbuf);
   nghttp3_buf_reset(&ebuf);
 
-  rv = nghttp3_qpack_encoder_set_max_dtable_size(&enc, 0);
+  rv = nghttp3_qpack_encoder_set_max_dtable_capacity(&enc, 0);
 
   CU_ASSERT(0 == rv);
 
-  rv = nghttp3_qpack_encoder_set_max_dtable_size(&enc, 1024);
+  rv = nghttp3_qpack_encoder_set_max_dtable_capacity(&enc, 1024);
 
   CU_ASSERT(0 == rv);
   CU_ASSERT(0 == enc.min_dtable_update);
@@ -518,7 +530,7 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
             enc.ctx.dtable_size);
   CU_ASSERT(SIZE_MAX == enc.min_dtable_update);
   CU_ASSERT(1024 == enc.last_max_dtable_update);
-  CU_ASSERT(1024 == enc.ctx.max_dtable_size);
+  CU_ASSERT(1024 == enc.ctx.max_dtable_capacity);
 
   nread = nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos,
                                              nghttp3_buf_len(&ebuf));
@@ -527,7 +539,7 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
   CU_ASSERT(2 == dec.ctx.next_absidx);
   CU_ASSERT(strlen("date") + strlen("bar1") + NGHTTP3_QPACK_ENTRY_OVERHEAD ==
             dec.ctx.dtable_size);
-  CU_ASSERT(1024 == dec.ctx.max_dtable_size);
+  CU_ASSERT(1024 == dec.ctx.max_dtable_capacity);
 
   decode_header_block(&dec, &pbuf, &rbuf, 4, mem);
   nghttp3_buf_reset(&pbuf);
@@ -572,11 +584,14 @@ void test_nghttp3_qpack_decoder_feedback(void) {
 
   nghttp3_buf_reserve(&dbuf, 4096, mem);
 
-  rv = nghttp3_qpack_encoder_init(&enc, 4096, 2, mem);
+  rv = nghttp3_qpack_encoder_init(&enc, mem);
 
   CU_ASSERT(0 == rv);
 
-  nghttp3_qpack_encoder_set_max_dtable_size(&enc, 4096);
+  nghttp3_qpack_encoder_set_hard_max_dtable_capacity(&enc, 4096);
+  nghttp3_qpack_encoder_set_max_blocked_streams(&enc, 2);
+
+  nghttp3_qpack_encoder_set_max_dtable_capacity(&enc, 4096);
 
   rv = nghttp3_qpack_decoder_init(&dec, 4096, 2, mem);
 
@@ -586,13 +601,13 @@ void test_nghttp3_qpack_decoder_feedback(void) {
                                     nghttp3_arraylen(nva1));
 
   CU_ASSERT(0 == rv);
-  CU_ASSERT(1 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(1 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
   rv = nghttp3_qpack_encoder_encode(&enc, &pbuf2, &rbuf2, &ebuf, 4, nva2,
                                     nghttp3_arraylen(nva2));
 
   CU_ASSERT(0 == rv);
-  CU_ASSERT(2 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(2 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
   nread = nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos,
                                              nghttp3_buf_len(&ebuf));
@@ -612,7 +627,7 @@ void test_nghttp3_qpack_decoder_feedback(void) {
   CU_ASSERT((nghttp3_ssize)nghttp3_buf_len(&dbuf) == nread);
   /* This will unblock all streams because higher insert count is
      acknowledged. */
-  CU_ASSERT(0 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(0 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
   CU_ASSERT(1 == nghttp3_map_size(&enc.streams));
   CU_ASSERT(1 == nghttp3_pq_size(&enc.min_cnts));
   CU_ASSERT(0 == nghttp3_ksl_len(&enc.blocked_streams));
@@ -638,7 +653,7 @@ void test_nghttp3_qpack_decoder_feedback(void) {
                                     nghttp3_arraylen(nva3));
 
   CU_ASSERT(0 == rv);
-  CU_ASSERT(1 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(1 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
   nread = nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos,
                                              nghttp3_buf_len(&ebuf));
@@ -655,7 +670,7 @@ void test_nghttp3_qpack_decoder_feedback(void) {
                                              nghttp3_buf_len(&dbuf));
 
   CU_ASSERT((nghttp3_ssize)nghttp3_buf_len(&dbuf) == nread);
-  CU_ASSERT(0 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(0 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
   CU_ASSERT(1 == nghttp3_map_size(&enc.streams));
   CU_ASSERT(3 == enc.krcnt);
 
@@ -673,7 +688,7 @@ void test_nghttp3_qpack_decoder_feedback(void) {
                                              nghttp3_buf_len(&dbuf));
 
   CU_ASSERT((nghttp3_ssize)nghttp3_buf_len(&dbuf) == nread);
-  CU_ASSERT(0 == nghttp3_qpack_encoder_get_num_blocked(&enc));
+  CU_ASSERT(0 == nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
   CU_ASSERT(0 == nghttp3_ksl_len(&enc.blocked_streams));
   CU_ASSERT(0 == nghttp3_pq_size(&enc.min_cnts));
   CU_ASSERT(0 == nghttp3_map_size(&enc.streams));
